//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "FingerDlgs.h"
#include "HelpDlgs.h"
#include "FileMan.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma link "FingCli"
#pragma resource "*.dfm"
#define MaxFingerCount 16
#define BUFSIZE 16384
TFingerDlg *FingerDlg;
//---------------------------------------------------------------------------
void __fastcall RunFingerTool(TForm *Owner)
{
   FingerDlg = new TFingerDlg(Owner);
   CenterDialog(FingerDlg,Owner);
   FingerDlg->ShowModal();
   delete FingerDlg;
}
//---------------------------------------------------------------------------
bool __fastcall GetFingerHistory(TStrings *tostrings)
{
   GetAppData(tostrings,FingerHistoryKey);
   if (!tostrings->Count) tostrings->CommaText = FingerHistory;
   return tostrings->Count>0;
}
//---------------------------------------------------------------------------
__fastcall TFingerDlg::TFingerDlg(TComponent* Owner)
	: TForm(Owner)
{
   GetFingerHistory(Query->Items);
   if (Query->Items->Count)
   {
      FingerBtn->Enabled = true;
      Query->ItemIndex = 0;
   }
   lines = new TStringList(); //RichEdit1->Lines;
   //results = new TMemoryStream();
   RichEdit1->Clear();
   editing = false;
   cap = Caption;
}
//---------------------------------------------------------------------------
__fastcall TFingerDlg::~TFingerDlg()
{
   //delete results;
   delete lines;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::CancelBtnClick(TObject *Sender)
{
   Timer1->Enabled = false;
   if (CancelBtn->Caption=="Cancel")
   {
      if (editing)
      {
         Query->Text = server;
         Query->Enabled = true;
         ActiveControl = Query;
         FingerBtn->Caption = "Finger";
         RichEdit1->ReadOnly = true;
         RichEdit1->Clear();
         Label1->Caption = "";
         editing = false;
         return;
      }
      FINGER->Abort();
      //results->SetSize(0);
      RichEdit1->Clear();
      RichEdit1->Lines->Add("Query cancelled!");
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
   }
   else ModalResult = mrOk;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::QueryChange(TObject *Sender)
{
   FingerBtn->Enabled = !Query->Text.IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FingerBtnClick(TObject *Sender)
{
   if (FingerBtn->Caption.Pos("Save")==1)
   {
      PutAppData(FingerHistoryKey,lines);
      Query->Items->Text = lines->Text;
      FingerBtn->Caption = "Finger";
      CancelBtn->Caption = "Close";
      Query->Enabled = true;
      Query->Text = "";
      if (Query->Items->Count) Query->ItemIndex = 0;
      ActiveControl = Query;
      Label1->Caption = "";
      RichEdit1->Clear();
      editing = false;
      return;
   }

   RichEdit1->Clear();
   AnsiString query = Query->Text;
   if (query.IsEmpty())
   {
      ShowMessage("No query string specified!");
      return;
   }

   TStrings *items = Query->Items;
   int idx = items->IndexOf(query);
   if (idx >= 0)
      items->Delete(idx);
   if (items->Count > 0)
      items->Insert(0,query);
   else
      items->Add(query);
   Query->ItemIndex = 0;

   if (query.Pos("@") == 0) query = "@"+query;
   FINGER->Query = query;
   ActiveControl = Panel1;
   //results->SetSize(0);
   CancelBtn->Caption = "Cancel";
   timeout = 0; RichEdit1->Lines->Add("Connecting"); Timer1->Enabled = true;
   FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = false;
   Label1->Caption = "";//Caption = cap;
   lines->Clear();
   FINGER->StartQuery();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERSessionConnected(TObject *Sender, WORD Error)
{
   if (Error==0)
   {
      RichEdit1->Lines->Text = "Receiving";
      timeout = 0;
   }
   else
   {
      Timer1->Enabled = false;
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      CancelBtn->Caption = "Close";
   }
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::AddLines(AnsiString &msg)
{
   AnsiString s;
   int start=1,stop=msg.Pos("\r");

   if (!stop) stop = msg.Length()+1;
   while (start <= msg.Length())
   {
      lines->Add(msg.SubString(start,stop-start));

      start = stop+1;
      if (start > msg.Length())
         break;

      while (msg[start] == '\n' || msg[start] == '\r')
         start++;

      stop = start;
      while (msg[stop] != '\r' && stop <= msg.Length())
         stop++;
   }
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERDataAvailable(TObject *Sender, WORD Error)
{
   int i,len;
   char d[2048];
   //sm(Error);
   if (Error)
   {
      switch (Error)
      {
         case WSAECONNREFUSED: RichEdit1->Lines->Add("No finger service available."); break;
         case WSAETIMEDOUT: RichEdit1->Lines->Add("Host unreachable."); break;
         case 11002: RichEdit1->Lines->Add("Host not found!"); break;
         default:
            RichEdit1->Lines->Add("Error ="+String(Error));
      }
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
      MyBeep(MB_OK);
      Abort();
      return;
   }
   len = FINGER->Receive(d,sizeof(d)-1);
   if (len > 1)
   {
      for (i = 0; i < len; i++)
      {
         if (String("\r\n\0").Pos(String(d[i])) < 1)
            incomingdata = incomingdata + d[i];
         if (d[i] == '\n')
         {
            AddLines(incomingdata+"\r");
            incomingdata = "";
         }
      }
   }
}
//---------------------------------------------------------------------------
/*void __fastcall TFingerDlg::FINGERDataAvailable(TObject *Sender, WORD Error)
{
   char *buf = new char[BUFSIZE];
   int len = FINGER->Receive(buf,BUFSIZE-1); buf[len] = '\0';
   results->Write(buf,len);
   delete buf;
}*/
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERQueryDone(TObject *Sender, WORD Error)
{
   SetRedraw(this,0);
   Timer1->Enabled = false;
   //sm(Error);
   if (Error)
   {
      switch (Error)
      {
         case WSAECONNREFUSED: RichEdit1->Lines->Add("No finger service available."); break;
         case WSAETIMEDOUT: RichEdit1->Lines->Add("Host unreachable."); break;
         case 11002: RichEdit1->Lines->Add("Host not found!"); break;
         default:
            RichEdit1->Lines->Add("Error ="+String(Error));
      }
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
      MyBeep(MB_OK);
   }
   else
   {
      Sleep(100);
      ActiveControl = RichEdit1;
      //results->Seek(0,soFromBeginning);
      //lines->LoadFromStream(results);
      SetRedraw(this,0);
      RichEdit1->Lines->Text = lines->Text;
      RichEdit1->SelStart = 0;
      lines->Clear();
      for (int i=0; i<3; i++)
         AppProcessMessages();
      SetRedraw(this,1);
      CancelBtn->Caption = "Close";
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      AppProcessMessages();
      MyBeep(MB_OK);

      AnsiString text = Query->Text;
      if (Query->ItemIndex>=0) Query->Items->Delete(Query->ItemIndex);
      Query->Items->Insert(0,text); Query->ItemIndex = 0;
      while (Query->Items->Count>MaxFingerCount)
         Query->Items->Delete(Query->Items->Count-1);
      PutAppData(FingerHistoryKey,Query->Items);
      Label1->Caption = s(RichEdit1->Lines->Count)+" items";
   }
   SetRedraw(this,1);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::SaveBtnClick(TObject *Sender)
{
   TSaveDialog *sd = SaveDialog1;
   if (sd->InitialDir.IsEmpty())
       sd->InitialDir = ExtractFilePath(Application->ExeName);
   sd->FileName = "*.txt";
   sd->Filter = "Text files (*.txt)|*.TXT";
   sd->Options << ofHideReadOnly;
   if (sd->Execute())
      RichEdit1->Lines->SaveToFile(SaveDialog1->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::Print1Click(TObject *Sender)
{
   RichEdit1->Print("Finger for "+Query->Text);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FormClose(TObject *Sender, TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::Timer1Timer(TObject *Sender)
{
   if (RichEdit1->Lines->Count)
      RichEdit1->Lines->Strings[0] = RichEdit1->Lines->Strings[0] + ".";
   if (timeout>=timeoutval*2)
   {
      Timer1->Enabled = false;
      FingerBtn->Enabled = false; HelpBtn->Enabled = Query->Enabled = false;
      FINGER->Abort();
      //results->SetSize(0);
      RichEdit1->Clear();
      RichEdit1->Lines->Add("Query timed out!");
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
      //Caption = cap;
      MyBeep(MB_OK);
   }
   timeout++;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::EditBtnClick(TObject *Sender)
{
   server = Query->Text;
   Query->Text = "Editing finger query list";
   CancelBtn->Caption = "Cancel";
   FingerBtn->Caption = "Save";
   lines->Text = Query->Items->Text;
   RichEdit1->ReadOnly = false;
   ActiveControl = RichEdit1;
   Query->Enabled = false;
   RichEdit1->SelStart = 0;
   editing = true;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopEditQueryListItemClick(TObject *Sender)
{
   EditBtnClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopupMenu1Popup(TObject *Sender)
{
   PopEditQueryListItem->Enabled = !editing;
   PopCopyItem->Enabled = RichEdit1->SelLength>0;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopCopyItemClick(TObject *Sender)
{
   RichEdit1->CopyToClipboard();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopSelectAllItemClick(TObject *Sender)
{
   RichEdit1->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::HelpBtnClick(TObject *Sender)
{
   OpenHelp("Finger","Finger Client","",NULL);
}
//---------------------------------------------------------------------------
